#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <math.h>
#include <stdio.h>
#include "ini.h"
#include "theme.h"

static SDL_Rect mkrect(int x,int y,int w,int h) {
    SDL_Rect tmp;
    tmp.x=x; tmp.y=y; tmp.w=w; tmp.h=h;
    return tmp;
}

void themeInitEngine(void) {
	TTF_Init();
}

static TTF_Font *themeLoadFontFromIni(iniHandle ini, char* section, char* path) {
	char fileName[1024];
	char buff[1024];
	int size;
	TTF_Font *ret;
	if (!iniGetStr(ini, section, "font", fileName)) exit(1);
	size=iniGetInt(ini, section, "size");
	strcpy(buff, path);
	strcat(buff, "/");
	strcat(buff, fileName);
	ret=TTF_OpenFont(buff, size);
	if (!ret) {
		fprintf(stderr, "Couldn't load font file %s: %s!\n",buff, TTF_GetError());
		exit(1);
	}
//	fprintf(stderr,"Loaded %s.\n", buff);
	return ret;
}

static SDL_Surface *themeLoadImageFromIni(iniHandle ini, char* section, char* path) {
	char fileName[1024];
	char buff[1024];
	SDL_Surface *ret;
	if (!iniGetStr(ini, section, "image", fileName)) exit(1);
	strcpy(buff, path);
	strcat(buff, "/");
	strcat(buff, fileName);
	ret=IMG_Load(buff);
	if (!ret) {
		fprintf(stderr, "Couldn't load image file %s!\n",fileName);
		exit(1);
	}
//	fprintf(stderr,"Loaded %s.\n", buff);
	return ret;
}


static void animLoad(iniHandle ini, char* section, char* path, themeAnimStruct *anim) {
	char sequence[1024];
	char name[1024];
	int x,n,picno;
	anim->pos.x=iniGetInt(ini, section, "posx");
	anim->pos.y=iniGetInt(ini, section, "posy");
	anim->loopStart=iniGetInt(ini, section, "loopstart");
	anim->loopEnd=iniGetInt(ini, section, "loopend");
	if (!iniGetStr(ini, section, "sequence", sequence)) exit(1);
	strcpy(name, path);
	strcat(name, "/");
	picno=0;
	for (x=0; x<strlen(sequence)+1; x++) {
		if (sequence[x]!=',' && sequence[x]!=0) {
			n=strlen(name);
			name[n]=sequence[x];
			name[n+1]=0;
		} else {
			//Try to load image in name.
			if (picno<THEME_MAX_PICS_IN_ANIM) {
				anim->pics[picno]=IMG_Load(name);
				if (!anim->pics[picno]) {
					fprintf(stderr, "Couldn't load image file %s!\n",name);
					exit(1);
				}
				picno++;
				strcpy(name, path);
				strcat(name, "/");
			}
		}
	}

	for (x=0; x<THEME_MAX_PICS_IN_ANIM; x++) anim->delays[x]=0;

	//Get the delays
	iniGetStr(ini, section, "delay", sequence);
	x=0;
	char *p1, *p2;
	p1=sequence;
	while(p1<(sequence+strlen(sequence))) {
		anim->delays[x]=strtol(p1, &p2, 10);
		x++;
		p1=p2+1; //skip past comma
	}


	anim->show=0;
	anim->state=0;
	anim->delay=0;
	anim->noPics=picno;
}

static void animShow(SDL_Surface *screen, themeAnimStruct *anim) {
	if (anim->show==0 && anim->state==0) return;
	if (anim->delay>0) {
		anim->delay--;
	} else {
		anim->state++;
		if (anim->show && anim->state==anim->loopEnd+1) anim->state=anim->loopStart;
		if (anim->state>anim->noPics) {
			anim->state=0;
			return;
		}
		anim->delay=anim->delays[anim->state-1];
	}
	int no=anim->state-1;
	SDL_Rect r;
	r.h=anim->pics[no]->h;
	r.w=anim->pics[no]->w;
	r.x=anim->pos.x;
	r.y=anim->pos.y;
	SDL_BlitSurface(anim->pics[no], NULL, screen, &r);
}

static int animIsStable(themeAnimStruct *anim) {
	if (anim->state<anim->loopStart) return 0;
	if (anim->state>anim->loopEnd) return 0;
	return 1;
}

static SDL_Color hexToColor(char* col) {
	int n=strtol(col, NULL, 16);
	SDL_Color ret={(n>>16)&0xff, (n>>8)&0xff, (n)&0xff};
	return ret;
}


void themeOptimizeGfx(themeHandle th) {
	SDL_DisplayFormatAlpha(th->bgndImage);
	SDL_DisplayFormatAlpha(th->fgndImage);
	SDL_DisplayFormatAlpha(th->ballImage);
	SDL_SetAlpha(th->bgndImage,SDL_RLEACCEL|SDL_SRCALPHA,255);
	SDL_SetAlpha(th->fgndImage,SDL_RLEACCEL|SDL_SRCALPHA,255);
	SDL_SetAlpha(th->ballImage,SDL_RLEACCEL|SDL_SRCALPHA,255);
}

	
themeHandle themeLoad(char *dir) {
	char buff[1024];
	int x;
	themeStruct *th;
	th=malloc(sizeof(themeStruct));
	strcpy(buff, dir);
	strcat(buff, "/theme.ini");
	iniHandle ini=iniOpenFile(buff);
	if (!ini) {
		fprintf(stderr, "Couldn't open theme ini file %s!\n", buff);
		exit(1);
	}

	th->bgndImage=themeLoadImageFromIni(ini, "bgnd", dir);
	th->fgndImage=themeLoadImageFromIni(ini, "fgnd", dir);
	th->ballImage=themeLoadImageFromIni(ini, "ball", dir);

	th->ballStart.x=iniGetInt(ini, "ball", "startx");
	th->ballStart.y=iniGetInt(ini, "ball", "starty");
	th->ballEnd.x=iniGetInt(ini, "ball", "endx");
	th->ballEnd.y=iniGetInt(ini, "ball", "endy");

	th->menuTextStart.x=iniGetInt(ini, "MenuText", "startx");
	th->menuTextStart.y=iniGetInt(ini, "MenuText", "starty");
	th->menuTextEnd.x=iniGetInt(ini, "MenuText", "endx");
	th->menuTextEnd.y=iniGetInt(ini, "MenuText", "endy");
	th->textStride=iniGetInt(ini, "MenuText", "stride");
	th->menuMargin=iniGetInt(ini, "MenuText", "margin");
	th->menuFont=themeLoadFontFromIni(ini, "MenuText", dir);

	th->headerTextStart.x=iniGetInt(ini, "HeaderText", "startx");
	th->headerTextStart.y=iniGetInt(ini, "HeaderText", "starty");
	th->headerTextEnd.x=iniGetInt(ini, "HeaderText", "endx");
	th->headerTextEnd.y=iniGetInt(ini, "HeaderText", "endy");
	th->headerFont=themeLoadFontFromIni(ini, "HeaderText", dir);

	iniGetStr(ini, "HeaderText", "color", buff);
	th->headerTextColor=hexToColor(buff);
	iniGetStr(ini, "MenuText", "color", buff);
	th->menuTextColor=hexToColor(buff);

	animLoad(ini, "wait", dir, &th->wait);

	animLoad(ini, "search", dir, &th->search);
	th->searchTextStart.x=iniGetInt(ini, "Search", "textstartx");
	th->searchTextStart.y=iniGetInt(ini, "Search", "textstarty");
	th->searchTextEnd.x=iniGetInt(ini, "Search", "textendx");
	th->searchTextEnd.y=iniGetInt(ini, "Search", "textendy");
	iniGetStr(ini, "Search", "color", buff);
	th->searchTextColor=hexToColor(buff);
	th->searchFont=themeLoadFontFromIni(ini, "Search", dir);

	animLoad(ini, "input", dir, &th->inputbox);
	th->inputboxTextStart.x=iniGetInt(ini, "Input", "textstartx");
	th->inputboxTextStart.y=iniGetInt(ini, "Input", "textstarty");
	th->inputboxTextEnd.x=iniGetInt(ini, "Input", "textendx");
	th->inputboxTextEnd.y=iniGetInt(ini, "Input", "textendy");
	iniGetStr(ini, "Input", "color", buff);
	th->inputboxTextColor=hexToColor(buff);
	th->inputboxFont=themeLoadFontFromIni(ini, "Input", dir);


	th->inputTitleStart.x=iniGetInt(ini, "InputTitle", "textstartx");
	th->inputTitleStart.y=iniGetInt(ini, "InputTitle", "textstarty");
	iniGetStr(ini, "InputTitle", "color", buff);
	th->inputTitleColor=hexToColor(buff);
	th->inputTitleFont=themeLoadFontFromIni(ini, "InputTitle", dir);

	iniCloseFile(ini);

	th->searchTextImg=NULL;
	th->inputboxTextImg=NULL;
	th->inputTitleImg=NULL;
	th->textPos=0;
	th->lastFadedItem=-1;
	th->cache.headerImage=NULL;
	for (x=0; x<NO_CACHED_ITEMS; x++) th->cache.image[x]=NULL;
	return th;
}

void themeNewMenu(themeHandle theme, char *header) {
	int x;
	if (theme->cache.headerImage) {
		SDL_FreeSurface(theme->cache.headerImage);
		theme->cache.headerImage=NULL;
	}
	theme->cache.headerImage=TTF_RenderUTF8_Blended(theme->headerFont, header, theme->headerTextColor);
	if (theme->cache.headerImage!=NULL) {
		SDL_DisplayFormatAlpha(theme->cache.headerImage);
	} else {
		fprintf(stderr, "Couldn't render header text %s!\n",header);
	}
	for (x=0; x<NO_CACHED_ITEMS; x++) {
		if (theme->cache.image[x]) SDL_FreeSurface(theme->cache.image[x]);
		theme->cache.image[x]=NULL;
	}
	theme->marqueePos=0;
	theme->lastFadedItem=-1;
}

void themeGetSize(themeHandle theme, vector *size) {
	size->x=theme->bgndImage->w;
	size->y=theme->bgndImage->h;
}

void themeFree(themeHandle theme) {
	//ToDo... theoretically. We only allocate a theme on startup, so no
	//memory is leaked by not having a free function.
}


SDL_Surface *themeGetImgFor(themeHandle theme, char *menuText) {
	//See if we have this in cache.
	int x;
	for (x=0; x<NO_CACHED_ITEMS; x++) {
		if (theme->cache.text[x]==menuText && theme->cache.image[x]!=NULL) {
			theme->cache.serial[x]=theme->cache.currSerial++;
			return theme->cache.image[x];
		}
	}
	//Uh-oh, image isn't found in cache. Guess we'll have to make a new one.
	//First, find a free slot.
	int n;
	n=0;
	for (x=0; x<NO_CACHED_ITEMS; x++) {
		if (theme->cache.serial[x]<theme->cache.serial[n]) {
			n=x;
		}
	}
	if (theme->cache.image[n]!=NULL) SDL_FreeSurface(theme->cache.image[n]);
	theme->cache.image[n]=TTF_RenderUTF8_Blended(theme->menuFont, menuText, theme->menuTextColor);
	SDL_DisplayFormatAlpha(theme->cache.image[n]);
	SDL_SetAlpha(theme->cache.image[n],SDL_RLEACCEL|SDL_SRCALPHA,255);
	theme->cache.serial[n]=theme->cache.currSerial++;
	theme->cache.text[n]=menuText;
//	printf("Created image, replacing ser %i at slot %i, text %p.\n",theme->cache.serial[n],n, menuText);
	return theme->cache.image[n];
}

#define UPDOWNSPEED	.2
#define LRSPEED	1
#define FADETIME 500

void themeDraw(SDL_Surface *screen, themeHandle theme, char **menuText, int menuItems, int selectedItem, float menuOffset, int dir) {
	int i, x;
	
	//Put new menu items into faders
	if (menuItems>theme->lastFadedItem) {
		//Move faded items to make place for new one
		int placesToMove=menuItems-theme->lastFadedItem;
		for (x=THEME_FADERS-1; x>=0; x--) {
			if (x>placesToMove) {
				theme->fadeTime[x]=theme->fadeTime[x-placesToMove];
			} else {
				theme->fadeTime[x]=SDL_GetTicks();
			}
		}
	}
	theme->lastFadedItem=menuItems;
	
	//Handle movement
	float dist=(float)selectedItem-theme->textPos;
	if ((dist<.05) && (dist>-.05)) {
		//No movenemt anymore :)
		theme->textPos=selectedItem;
		theme->textScrollPos+=theme->textScrollDir;
		if (theme->textScrollPos<=0) theme->textScrollDir=LRSPEED;
	} else {
		//Move.
		theme->textScrollPos=0;
		theme->textScrollDir=1;
		theme->textPos+=(dist*UPDOWNSPEED);
		theme->marqueePos=0;
	}

	//Blit the background.
	SDL_BlitSurface(theme->bgndImage, NULL, screen, NULL);
	//Blit the text
	//First set clipping mask so we won't draw outside of the text screen.
	SDL_Rect sr;
	sr.x=theme->menuTextStart.x;
	sr.y=theme->menuTextStart.y;
	sr.w=theme->menuTextEnd.x-theme->menuTextStart.x;
	sr.h=theme->menuTextEnd.y-theme->menuTextStart.y;
	SDL_SetClipRect(screen, &sr);
	
	//Calc where the selected items top left should be drawn.
	int selY=(theme->menuTextEnd.y+theme->menuTextStart.y)/2-theme->textStride/2;//-(theme->textPos*theme->textStride);
	for (i=0; i<menuItems; i++) {
		int thisY=selY+theme->textStride*((float)i-theme->textPos);
		//See if this iten should be drawn.
		if (thisY<theme->menuTextEnd.y && thisY>(theme->menuTextStart.y-theme->textStride*2)) {
			SDL_Surface *textIm=themeGetImgFor(theme, menuText[i]);
			if (textIm!=NULL) {
				SDL_Rect r, r2;
				r.x=theme->menuTextStart.x+(menuOffset*(theme->menuTextEnd.x-theme->menuTextStart.x));
				r.w=theme->menuTextEnd.x-theme->menuTextStart.x;
				r.y=thisY;
				r.h=theme->textStride;
				r2.x=-theme->menuMargin; r2.y=0;
				r2.h=textIm->h; r2.w=textIm->w+theme->menuMargin;
				int isMovingAlready=0;
				if (i>menuItems-THEME_FADERS) {
					//Oooh, it's new. Move in place dependant on the time it exists.
					//The planning used to be a fade, but that won't work with stupid SDL :/
					long amount=FADETIME-(SDL_GetTicks()-theme->fadeTime[menuItems-i]);
					if (amount>0) {
						//Get quadratic movement.
						amount=(amount*amount*(theme->menuTextEnd.x-theme->menuTextStart.x))/(FADETIME*FADETIME);
						r.x+=amount*dir;
						isMovingAlready=1;
					} 
				} 
				//Not new and selected. Do the marquee effect if needed.
				if (!isMovingAlready && fabs((float)i-theme->textPos)<0.05) {
					if (r.w<textIm->w) {
						theme->marqueePos++;
						if (r.w>textIm->w-theme->marqueePos+(theme->menuMargin*2)) {
							theme->marqueePos=-theme->marqueePos;
						}
						r2.x+=abs(theme->marqueePos);
					}
				}
				SDL_BlitSurface(textIm, &r2, screen, &r);
			}

		}
	}

	//Undo clipping mask
	SDL_SetClipRect(screen, NULL);


	//Blit header text
	SDL_Rect r=mkrect(theme->headerTextStart.x, theme->headerTextStart.y, theme->headerTextEnd.x-theme->headerTextStart.x, theme->headerTextEnd.y-theme->headerTextStart.y);
	SDL_BlitSurface(theme->cache.headerImage, NULL, screen, &r);

	//Blit foreground
	SDL_BlitSurface(theme->fgndImage, NULL, screen, NULL);

	//Place the 'ball'
	if (menuItems>0) {
		float ballProgress=theme->textPos/menuItems;
		if (ballProgress<0) ballProgress=0;
		if (ballProgress>1) ballProgress=1;
		SDL_Rect r;
		r.h=theme->ballImage->h;
		r.w=theme->ballImage->w;
		r.x=(theme->ballStart.x*(1-ballProgress)+theme->ballEnd.x*ballProgress)-r.w/2;
		r.y=(theme->ballStart.y*(1-ballProgress)+theme->ballEnd.y*ballProgress)-r.h/2;
		SDL_BlitSurface(theme->ballImage, NULL, screen, &r);
	}

	//Handle wait anim
	animShow(screen, &theme->wait);
	
	//Show search anim and text, if needed
	animShow(screen, &theme->search);
	if (theme->searchTextImg!=NULL && animIsStable(&theme->search)) {
		SDL_Rect r1=mkrect(theme->searchTextStart.x, theme->searchTextStart.y, theme->searchTextEnd.x-theme->searchTextStart.x, theme->searchTextEnd.y-theme->searchTextStart.y);
		SDL_Rect r2=mkrect(0, 0, theme->searchTextImg->w, theme->searchTextImg->h);
		if (r2.w>r1.w) {
			r2.x=r2.w-r1.w;
			r2.w=r1.w;
		}
		SDL_BlitSurface(theme->searchTextImg, &r2, screen, &r1);
	}

	//Handle remaining input anim stuff, if needed.
	theme->inputbox.show=0;
	animShow(screen, &theme->inputbox);
}


void themeDrawInputbox(SDL_Surface *screen, themeHandle theme) {
	//Blit the background.
	SDL_BlitSurface(theme->bgndImage, NULL, screen, NULL);
	//Blit foreground
	SDL_BlitSurface(theme->fgndImage, NULL, screen, NULL);
	//Show anim of input box
	theme->inputbox.show=1;
	animShow(screen, &theme->inputbox);
	if (animIsStable(&theme->inputbox)) {
		//Blit header text
		SDL_Rect r;
		r.h=theme->inputTitleImg->h;
		r.w=theme->inputTitleImg->w;
		r.x=theme->inputTitleStart.x;
		r.y=theme->inputTitleStart.y;
		SDL_BlitSurface(theme->inputTitleImg, NULL, screen, &r);
		//Blit input text
		if (theme->inputboxTextImg!=NULL) {
			SDL_Rect r1=mkrect(theme->inputboxTextStart.x, theme->inputboxTextStart.y, theme->inputboxTextEnd.x-theme->inputboxTextStart.x, theme->inputboxTextEnd.y-theme->inputboxTextStart.y);
			SDL_Rect r2=mkrect(0, 0, theme->inputboxTextImg->w, theme->inputboxTextImg->h);
			if (r2.w>r1.w) {
				r2.x=r2.w-r1.w;
				r2.w=r1.w;
			}
			SDL_BlitSurface(theme->inputboxTextImg, &r2, screen, &r1);
		}
	}
}

void themeSetInputboxTitle(themeHandle th, char *title) {
	if (strlen(title)==0) strcpy(title,"?");
	if (th->inputTitleImg!=NULL) SDL_FreeSurface(th->inputTitleImg);
	th->inputTitleImg=TTF_RenderUTF8_Blended(th->inputTitleFont, title, th->inputTitleColor);
	SDL_DisplayFormatAlpha(th->inputTitleImg);
	SDL_SetAlpha(th->inputTitleImg,SDL_RLEACCEL|SDL_SRCALPHA,255);
}

void themeSetInputbox(themeHandle theme, char *input, char cursorPos) {
	//ToDo: Cursor handling
	if (theme->inputboxTextImg!=NULL) {
		SDL_FreeSurface(theme->inputboxTextImg);
	}
	theme->inputboxTextImg=NULL;
	if (strlen(input)==0) return;
	theme->inputboxTextImg=TTF_RenderUTF8_Blended(theme->inputboxFont, input, theme->inputboxTextColor);
	SDL_DisplayFormatAlpha(theme->inputboxTextImg);
	SDL_SetAlpha(theme->inputboxTextImg,SDL_RLEACCEL|SDL_SRCALPHA,255);
}


int themeGetPageSize(themeHandle theme) {
	return ((theme->menuTextEnd.y-theme->menuTextStart.y)/theme->textStride)-1;
}

void themeSetWait(themeHandle theme, int state) {
	theme->wait.show=state;
}

void themeSetSearch(themeHandle theme, char *text) {
	theme->search.show=1;
	if (theme->searchTextImg!=NULL) SDL_FreeSurface(theme->searchTextImg);
	if (strlen(text)!=0) {
		theme->searchTextImg=TTF_RenderUTF8_Blended(theme->searchFont, text, theme->searchTextColor);
		SDL_DisplayFormatAlpha(theme->searchTextImg);
		SDL_SetAlpha(theme->searchTextImg,SDL_RLEACCEL|SDL_SRCALPHA,255);
		theme->search.show=1;
	} else {
		theme->searchTextImg=NULL;
		theme->search.show=0;
	}
}
