#ifndef THEME_H
#define THEME_H
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

typedef struct {
	int x;
	int y;
} vector;


//Faders make the effect of new items 'scrolling in'. It was first supposed to
//fade 'em in, but SDL has trouble with alpha-mapping channels with an alpha channel...
#define THEME_FADERS 20

//Number of cached images. Make this at least as big as the number of text items on the screen.
#define NO_CACHED_ITEMS	32

typedef struct {
	SDL_Surface *image[NO_CACHED_ITEMS];
	int serial[NO_CACHED_ITEMS];
	int currSerial;
	SDL_Surface *headerImage;
	char *text[NO_CACHED_ITEMS];
} themeImageCache;

#define THEME_MAX_PICS_IN_ANIM 20
typedef struct {
	int show;
	int state;
	vector pos;
	int loopStart;
	int loopEnd;
	int noPics;
	int delay;
	int delays[THEME_MAX_PICS_IN_ANIM];
	SDL_Surface *pics[THEME_MAX_PICS_IN_ANIM];
} themeAnimStruct;

typedef struct {
	SDL_Surface *bgndImage;
	SDL_Surface *fgndImage;
	SDL_Surface *ballImage;
	vector menuTextStart;
	vector menuTextEnd;
	vector headerTextStart;
	vector headerTextEnd;
	int menuMargin;
	int textStride;
	vector ballStart;
	vector ballEnd;
	TTF_Font *menuFont;
	TTF_Font *headerFont;
	SDL_Color menuTextColor;
	SDL_Color headerTextColor;
	float textPos;
	int textScrollPos;
	int textScrollDir;
	int textPosTarget;
	char headerText[512];
	long fadeTime[THEME_FADERS];
	int lastFadedItem;
	themeImageCache cache;
	themeAnimStruct wait;
	themeAnimStruct search;
	vector searchTextStart;
	vector searchTextEnd;
	TTF_Font *searchFont;
	SDL_Color searchTextColor;
	SDL_Surface *searchTextImg;
	themeAnimStruct inputbox;
	vector inputboxTextStart;
	vector inputboxTextEnd;
	TTF_Font *inputboxFont;
	SDL_Color inputboxTextColor;
	SDL_Surface *inputboxTextImg;
	vector inputTitleStart;
	TTF_Font *inputTitleFont;
	SDL_Color inputTitleColor;
	SDL_Surface *inputTitleImg;
	int marqueePos;
} themeStruct;

typedef themeStruct *themeHandle;

void themeInitEngine(void);
themeHandle themeLoad(char *dir);
void themeFree(themeHandle theme);
void themeSetPos(int pos);
void themeDraw(SDL_Surface *screen, themeHandle theme, char **menuText, int menuItems, int selectedItem, float menuOffset, int dir);
void themeNewMenu(themeHandle theme, char *header);
int themeGetPageSize(themeHandle theme);
void themeSetWait(themeHandle theme, int state);
void themeSetSearch(themeHandle theme, char *search);
void themeDrawInputbox(SDL_Surface *screen, themeHandle theme);
void themeSetInputbox(themeHandle theme, char *input, char cursorPos);
void themeSetInputboxTitle(themeHandle th, char *title);
void themeOptimizeGfx(themeHandle th);
void themeGetSize(themeHandle theme, vector *size);

#endif